@@ -14,7 +14,6 @@
 #include <QHash>
 #include <QDBusMessage>
 #include <QStringBuilder>
-#include <chrono>
 #include <unistd.h>
 
 ApplicationManager1Service::~ApplicationManager1Service() = default;
@@ -25,8 +24,9 @@ ApplicationManager1Service::ApplicationManager1Service(std::unique_ptr<Identifie
     , m_storage(std::move(storage))
 {
     using namespace std::chrono_literals;
-    m_reloadTimer.setInterval(1s);
-    connect(&m_reloadTimer, &QTimer::timeout, [this] { m_reloadTimer.stop(); });
+    m_reloadTimer.setInterval(50);
+    m_reloadTimer.setSingleShot(true);
+    connect(&m_reloadTimer, &QTimer::timeout, this, &ApplicationManager1Service::doReloadApplications);
 }
 
 void ApplicationManager1Service::initService(QDBusConnection &connection) noexcept
@@ -59,6 +59,10 @@ void ApplicationManager1Service::initService(QDBusConnection &connection) noexce
         std::terminate();
     }
 
+    if (m_compatibilityManager.reset(new (std::nothrow) CompatibilityManager()); !m_compatibilityManager) {
+        qWarning() << "new CompatibilityManager failed.";
+    }
+
     connect(&m_watcher, &QFileSystemWatcher::directoryChanged, this, &ApplicationManager1Service::ReloadApplications);
     auto unhandled = m_watcher.addPaths(getDesktopFileDirs());
     for (const auto &dir : unhandled) {
@@ -68,6 +72,31 @@ void ApplicationManager1Service::initService(QDBusConnection &connection) noexce
     auto &dispatcher = SystemdSignalDispatcher::instance();
 
     connect(&dispatcher, &SystemdSignalDispatcher::SystemdUnitNew, this, &ApplicationManager1Service::addInstanceToApplication);
+    connect(
+        &dispatcher, &SystemdSignalDispatcher::SystemdJobNew, this, [this](QString unitName, QDBusObjectPath systemdUnitPath) {
+            auto info = processUnitName(unitName);
+            auto appId = std::move(info.applicationID);
+
+            if (appId.isEmpty()) {
+                return;
+            }
+
+            auto app = m_applicationList.value(appId);
+
+            if (!app) {
+                return;
+            }
+
+            // 服务在 AM 之后启动那么 instance size 是 0， newJob 时尝试添加一次
+            // 比如 dde-file-manager.service 如果启动的比 AM 晚，那么在 scanInstances 时不会 addInstanceToApplication
+            if (app->instances().size() > 0) {
+                return;
+            }
+
+            qDebug() << "add Instance " << unitName << "on JobNew, " << app->instances().size();
+
+            addInstanceToApplication(unitName, systemdUnitPath);
+        });
 
     connect(&dispatcher,
             &SystemdSignalDispatcher::SystemdUnitRemoved,
@@ -143,53 +172,55 @@ void ApplicationManager1Service::initService(QDBusConnection &connection) noexce
     if (flag.open(QFile::WriteOnly | QFile::Truncate)) {
         flag.write(sessionId, sessionId.size());
     }
-
-    constexpr auto XSettings = "org.deepin.dde.XSettings1";
-
-    auto *watcher = new (std::nothrow)
-        QDBusServiceWatcher{XSettings, QDBusConnection::sessionBus(), QDBusServiceWatcher::WatchForRegistration, this};
-
-    auto *sigCon = new (std::nothrow) QMetaObject::Connection{};
-
-    auto singleSlot = [watcher, sigCon, autostartMap = std::move(needLaunch)]() {
-        QObject::disconnect(*sigCon);
-        delete sigCon;
-        qDebug() << XSettings << "is registered.";
-
-        for (const auto &[app, realExec] : autostartMap.asKeyValueRange()) {
+    auto value = QString::fromLocal8Bit(qgetenv("XDG_SESSION_TYPE"));
+    if (!value.isEmpty() && value == QString::fromLocal8Bit("wayland")) {
+        for (const auto &[app, realExec] : needLaunch.asKeyValueRange()) {
             app->Launch({}, {}, {}, realExec);
         }
+    } else {
+        constexpr auto XSettings = "org.deepin.dde.XSettings1";
 
-        watcher->deleteLater();
-    };
+        auto *watcher = new (std::nothrow)
+            QDBusServiceWatcher{XSettings, QDBusConnection::sessionBus(), QDBusServiceWatcher::WatchForRegistration, this};
 
-    if (watcher != nullptr) {
-        *sigCon = connect(watcher, &QDBusServiceWatcher::serviceRegistered, singleSlot);
-    }
+        auto *sigCon = new (std::nothrow) QMetaObject::Connection{};
 
-    auto msg =
-        QDBusMessage::createMethodCall("org.freedesktop.DBus", "/org/freedesktop/DBus", "org.freedesktop.DBus", "NameHasOwner");
-    msg << XSettings;
+        auto singleSlot = [watcher, sigCon, autostartMap = std::move(needLaunch)]() {
+            QObject::disconnect(*sigCon);
+            delete sigCon;
+            qDebug() << XSettings << "is registered.";
 
-    auto reply = QDBusConnection::sessionBus().call(msg);
-    if (reply.type() != QDBusMessage::ReplyMessage) {
-        qWarning() << "call org.freedesktop.DBus::NameHasOwner failed, skip autostart:" << reply.errorMessage();
-        // The connection should not be deleted, failure to call org.freedesktop.DBus::NameHasOwner does not mean that the
-        // XSettings service is invalid.
-        return;
-    }
+            for (const auto &[app, realExec] : autostartMap.asKeyValueRange()) {
+                app->Launch({}, {}, {}, realExec);
+            }
 
-    if (reply.arguments().first().toBool()) {
-        singleSlot();
+            watcher->deleteLater();
+        };
+
+        if (watcher != nullptr) {
+            *sigCon = connect(watcher, &QDBusServiceWatcher::serviceRegistered, singleSlot);
+        }
+
+        auto msg = QDBusMessage::createMethodCall(
+            "org.freedesktop.DBus", "/org/freedesktop/DBus", "org.freedesktop.DBus", "NameHasOwner");
+        msg << XSettings;
+
+        auto reply = QDBusConnection::sessionBus().call(msg);
+        if (reply.type() != QDBusMessage::ReplyMessage) {
+            qWarning() << "call org.freedesktop.DBus::NameHasOwner failed, skip autostart:" << reply.errorMessage();
+            // The connection should not be deleted, failure to call org.freedesktop.DBus::NameHasOwner does not mean that the
+            // XSettings service is invalid.
+            return;
+        }
+
+        if (reply.arguments().first().toBool()) {
+            singleSlot();
+        }
     }
 }
 
 void ApplicationManager1Service::addInstanceToApplication(const QString &unitName, const QDBusObjectPath &systemdUnitPath)
 {
-    if (!isApplication(systemdUnitPath)) {
-        return;
-    }
-
     auto info = processUnitName(unitName);
     auto appId = std::move(info.applicationID);
     auto launcher = std::move(info.Launcher);
@@ -198,31 +229,28 @@ void ApplicationManager1Service::addInstanceToApplication(const QString &unitNam
     if (appId.isEmpty()) {
         return;
     }
+    if (instanceId.isEmpty()) {
+        instanceId = QUuid::createUuid().toString(QUuid::Id128);
+    }
 
-    auto appIt = std::find_if(m_applicationList.cbegin(),
-                              m_applicationList.cend(),
-                              [&appId](const QSharedPointer<ApplicationService> &app) { return app->id() == appId; });
+    auto app = m_applicationList.value(appId);
 
-    if (appIt == m_applicationList.cend()) {
+    if (!app) {
         qWarning() << "couldn't find app" << appId << "in application manager.";
         return;
     }
 
-    (*appIt)->updateAfterLaunch(sender() != nullptr);  // activate by signal
+    app->updateAfterLaunch(sender() != nullptr);  // activate by signal
 
-    const auto &applicationPath = (*appIt)->applicationPath().path();
+    const auto &applicationPath = app->applicationPath().path();
 
-    if (!(*appIt)->addOneInstance(instanceId, applicationPath, systemdUnitPath.path(), launcher)) {
+    if (!app->addOneInstance(instanceId, applicationPath, systemdUnitPath.path(), launcher)) {
         qCritical() << "add Instance failed:" << applicationPath << unitName << systemdUnitPath.path();
     }
 }
 
 void ApplicationManager1Service::removeInstanceFromApplication(const QString &unitName, const QDBusObjectPath &systemdUnitPath)
 {
-    if (!isApplication(systemdUnitPath)) {
-        return;
-    }
-
     auto info = processUnitName(unitName);
     auto appId = std::move(info.applicationID);
     auto launcher = std::move(info.Launcher);
@@ -232,16 +260,14 @@ void ApplicationManager1Service::removeInstanceFromApplication(const QString &un
         return;
     }
 
-    auto appIt = std::find_if(m_applicationList.cbegin(),
-                              m_applicationList.cend(),
-                              [&appId](const QSharedPointer<ApplicationService> &app) { return app->id() == appId; });
+    auto app = m_applicationList.value(appId);
 
-    if (appIt == m_applicationList.cend()) {
+    if (!app) {
         qWarning() << "couldn't find app" << appId << "in application manager.";
         return;
     }
 
-    const auto &appIns = (*appIt)->applicationInstances();
+    const auto &appIns = app->applicationInstances();
 
     auto instanceIt =
         std::find_if(appIns.cbegin(), appIns.cend(), [&systemdUnitPath](const QSharedPointer<InstanceService> &value) {
@@ -249,7 +275,7 @@ void ApplicationManager1Service::removeInstanceFromApplication(const QString &un
         });
 
     if (instanceIt != appIns.cend()) {
-        (*appIt)->removeOneInstance(instanceIt.key());
+        app->removeOneInstance(instanceIt.key());
         return;
     }
 
@@ -304,7 +330,12 @@ void ApplicationManager1Service::scanApplications() noexcept
 void ApplicationManager1Service::scanInstances() noexcept
 {
     auto &conn = ApplicationManager1DBus::instance().globalDestBus();
-    auto call_message = QDBusMessage::createMethodCall(SystemdService, SystemdObjectPath, SystemdInterfaceName, "ListUnits");
+    auto call_message =
+        QDBusMessage::createMethodCall(SystemdService, SystemdObjectPath, SystemdInterfaceName, "ListUnitsByPatterns");
+    QList<QVariant> args;
+    args << QVariant::fromValue(QStringList{"running", "start"});
+    args << QVariant::fromValue(QStringList{"dde*", "deepin*"});
+    call_message.setArguments(args);
     auto result = conn.call(call_message);
     if (result.type() == QDBusMessage::ErrorMessage) {
         qCritical() << "failed to scan existing instances: call to ListUnits failed:" << result.errorMessage();
@@ -315,12 +346,7 @@ void ApplicationManager1Service::scanInstances() noexcept
     QList<SystemdUnitDBusMessage> units;
     v.value<QDBusArgument>() >> units;
     for (const auto &unit : units) {
-        if (!isApplication(unit.objectPath)) {
-            continue;
-        }
-        if (unit.subState == "running") {
-            this->addInstanceToApplication(unit.name, unit.objectPath);
-        }
+        this->addInstanceToApplication(unit.name, unit.objectPath);
     }
 }
 
@@ -349,7 +375,6 @@ QHash<QSharedPointer<ApplicationService>, QString> ApplicationManager1Service::s
 
     for (auto &it : autostartItems) {
         auto desktopFile = std::move(it.second);
-
         DesktopFileGuard guard{desktopFile};
         if (!guard.try_open()) {
             continue;
@@ -366,12 +391,15 @@ QHash<QSharedPointer<ApplicationService>, QString> ApplicationManager1Service::s
 
         QSharedPointer<ApplicationService> app{nullptr};
         auto asApplication = tmp.value(DesktopFileEntryKey, X_Deepin_GenerateSource).value_or(DesktopEntry::Value{});
+        auto shouldLaunch = tmp.value(DesktopFileEntryKey, DesktopEntryHidden).value_or(DesktopEntry::Value{});
+        auto autostartFlag = shouldLaunch.isNull() || (shouldLaunch.toString().compare("true", Qt::CaseInsensitive) != 0);
+
         if (!asApplication.isNull()) {  // modified by application manager
             auto appSource = asApplication.toString();
 
-            QFileInfo sourceInfo{appSource};
-            if ((!sourceInfo.exists() or !sourceInfo.isFile()) and !file.remove()) {
-                qWarning() << "remove invalid autostart file error:" << file.error();
+            const QFileInfo sourceInfo{appSource};
+            if (!sourceInfo.exists() || !sourceInfo.isFile()) {
+                qWarning() << "autostart file" << appSource << "doesn't exist, skip.";
                 continue;
             }
 
@@ -395,19 +423,15 @@ QHash<QSharedPointer<ApplicationService>, QString> ApplicationManager1Service::s
                 qWarning() << "add autostart application failed, skip.";
                 continue;
             }
-        }
 
-        auto shouldLaunch = tmp.value(DesktopFileEntryKey, DesktopEntryHidden).value_or(DesktopEntry::Value{});
-        if (!shouldLaunch.isNull() and (shouldLaunch.toString().compare("true", Qt::CaseInsensitive) == 0)) {
-            qInfo() << "shouldn't launch this autoStart item.";
-            continue;
-        }
-
-        if (app) {
-            auto realExec = tmp.value(DesktopFileEntryKey, "Exec").value_or(QString{""}).toString();
-            qInfo() << "launch normal autostart application " << app->id() << " by " << realExec;
-            ret.insert(app, realExec);
             app->setAutostartSource({desktopFile.sourcePath(), std::move(tmp)});
+            if (autostartFlag) {
+                auto val = tmp.value(DesktopFileEntryKey, "Exec").value_or(DesktopEntry::Value{});
+                auto realExec = val.isNull() ? QString{} : val.toString();
+                qInfo() << "launch autostart application " << app->id() << " by " << realExec;
+                ret.insert(app, realExec);
+            }
+
             continue;
         }
 
@@ -417,28 +441,32 @@ QHash<QSharedPointer<ApplicationService>, QString> ApplicationManager1Service::s
             continue;
         }
 
-        if (auto appIt = std::find_if(m_applicationList.cbegin(),
-                                      m_applicationList.cend(),
-                                      [&desktopFile](const auto &app) { return desktopFile.desktopId() == app->id(); });
-            appIt != m_applicationList.cend()) {
-            auto realExec = tmp.value(DesktopFileEntryKey, "Exec").value_or(QString{""}).toString();
-            qInfo() << "launch exist autostart application " << (*appIt)->id() << " by " << realExec;
-            ret.insert(*appIt, realExec);
-            (*appIt)->setAutostartSource({desktopFile.sourcePath(), std::move(tmp)});
+        if (auto existApp = m_applicationList.value(desktopFile.desktopId()); existApp) {
+            if (autostartFlag) {
+                auto val = tmp.value(DesktopFileEntryKey, "Exec").value_or(DesktopEntry::Value{});
+                auto realExec = val.isNull() ? QString{} : val.toString();
+                qInfo() << "launch exist autostart application " << existApp->id() << " by " << realExec;
+                ret.insert(existApp, realExec);
+            }
+
+            existApp->setAutostartSource({desktopFile.sourcePath(), std::move(tmp)});
             continue;
         }
 
         guard.close();
         // new application
-        auto newApp = addApplication(std::move(desktopFile));
-        if (!newApp) {
+        auto originalSource = desktopFile.sourcePath();
+        app = addApplication(std::move(desktopFile));
+        if (!app) {
             qWarning() << "add autostart application failed, skip.";
             continue;
         }
+        app->setAutostartSource({originalSource, {}});
 
-        qInfo() << "launch new autostart application " << newApp->id();
-        newApp->setAutostartSource({desktopFile.sourcePath(), {}});
-        ret.insert(newApp, {});
+        if (autostartFlag) {
+            qInfo() << "launch new autostart application " << app->id();
+            ret.insert(app, {});
+        }
     }
 
     return ret;
@@ -472,15 +500,17 @@ void ApplicationManager1Service::loadHooks() noexcept
 
 QList<QDBusObjectPath> ApplicationManager1Service::list() const
 {
-    return m_applicationList.keys();
+    QList<QDBusObjectPath> paths;
+    for (const auto &appId : m_applicationList.keys())
+        paths << QDBusObjectPath{getObjectPathFromAppId(appId)};
+
+    return paths;
 }
 
 QSharedPointer<ApplicationService> ApplicationManager1Service::addApplication(DesktopFile desktopFileSource) noexcept
 {
-    auto objectPath = QDBusObjectPath{getObjectPathFromAppId(desktopFileSource.desktopId())};
-    if (auto app = m_applicationList.constFind(objectPath); app != m_applicationList.cend()) {
-        qInfo() << "this application already exists."
-                << "current desktop source:" << desktopFileSource.sourcePath()
+    if (auto app = m_applicationList.constFind(desktopFileSource.desktopId()); app != m_applicationList.cend()) {
+        qInfo() << "this application already exists." << "current desktop source:" << desktopFileSource.sourcePath()
                 << "exists app source:" << app->data()->desktopFileSource().sourcePath();
         return *app;
     }
@@ -502,7 +532,7 @@ QSharedPointer<ApplicationService> ApplicationManager1Service::addApplication(De
     if (!registerObjectToDBus(ptr, application->applicationPath().path(), ApplicationInterface)) {
         return nullptr;
     }
-    m_applicationList.insert(application->applicationPath(), application);
+    m_applicationList.insert(application->id(), application);
 
     emit listChanged();
     emit InterfacesAdded(application->applicationPath(), getChildInterfacesAndPropertiesFromObject(ptr));
@@ -510,18 +540,19 @@ QSharedPointer<ApplicationService> ApplicationManager1Service::addApplication(De
     return application;
 }
 
-void ApplicationManager1Service::removeOneApplication(const QDBusObjectPath &application) noexcept
+void ApplicationManager1Service::removeOneApplication(const QString &appId) noexcept
 {
-    if (auto it = m_applicationList.find(application); it != m_applicationList.cend()) {
-        emit InterfacesRemoved(application, getChildInterfacesFromObject(it->data()));
+    auto objectPath = QDBusObjectPath{getObjectPathFromAppId(appId)};
+    if (auto it = m_applicationList.find(appId); it != m_applicationList.cend()) {
+        emit InterfacesRemoved(objectPath, getChildInterfacesFromObject(it->data()));
         if (auto ptr = m_storage.lock(); ptr) {
-            auto appId = (*it)->id();
             if (!ptr->deleteApplication(appId)) {
                 qCritical() << "failed to delete all properties of" << appId;
             }
         }
-        unregisterObjectFromDBus(application.path());
-        m_applicationList.remove(application);
+        unregisterObjectFromDBus(objectPath.path());
+        std::ignore = it->data()->RemoveFromDesktop();
+        m_applicationList.erase(it);
 
         emit listChanged();
     }
@@ -529,8 +560,8 @@ void ApplicationManager1Service::removeOneApplication(const QDBusObjectPath &app
 
 void ApplicationManager1Service::removeAllApplication() noexcept
 {
-    for (const auto &app : m_applicationList.keys()) {
-        removeOneApplication(app);
+    for (const auto &appId : m_applicationList.keys()) {
+        removeOneApplication(appId);
     }
 }
 
@@ -545,36 +576,33 @@ QString ApplicationManager1Service::Identify(const QDBusUnixFileDescriptor &pidf
 
     Q_ASSERT_X(static_cast<bool>(m_identifier), "Identify", "Broken Identifier.");
 
-    auto pid = getPidFromPidFd(pidfd);
-    if (pid == 0) {
-        sendErrorReply(QDBusError::Failed, "pid is invalid");
-        return {};
-    }
-
-    const auto ret = m_identifier->Identify(pid);
+    const auto ret = m_identifier->Identify(pidfd);
     if (ret.ApplicationId.isEmpty()) {
-        sendErrorReply(QDBusError::Failed, "Identify failed.");
+        safe_sendErrorReply(QDBusError::Failed, "Identify failed.");
         return {};
     }
 
-    auto app = std::find_if(m_applicationList.cbegin(), m_applicationList.cend(), [&ret](const auto &appPtr) {
-        return appPtr->id() == ret.ApplicationId;
-    });
-
-    if (app == m_applicationList.cend()) {
-        sendErrorReply(QDBusError::Failed, "can't find application:" % ret.ApplicationId);
+    if (!m_applicationList.contains(ret.ApplicationId)) {
+        safe_sendErrorReply(QDBusError::Failed, "can't find application:" % ret.ApplicationId);
         return {};
     }
 
-    auto instancePath = (*app)->findInstance(ret.InstanceId);
-
-    if (auto path = instancePath.path(); path.isEmpty()) {
-        sendErrorReply(QDBusError::Failed, "can't find instance:" % path);
+    auto app = m_applicationList.value(ret.ApplicationId);
+    QDBusObjectPath instancePath;
+    const auto &instances = app->instances();
+    if (ret.InstanceId.isEmpty() && instances.size() == 1) {
+        // Maybe a dbus systemd service
+        instancePath = instances.constFirst();
+    } else {
+        instancePath = app->findInstance(ret.InstanceId);
+    }
+    if (instancePath.path().isEmpty()) {
+        safe_sendErrorReply(QDBusError::Failed, "can't find instance:" % ret.InstanceId);
         return {};
     }
 
     instance = instancePath;
-    auto instanceObj = (*app)->m_Instances.constFind(instance);
+    auto instanceObj = app->m_Instances.constFind(instance);
     application_instance_info = getChildInterfacesAndPropertiesFromObject(instanceObj->get());
 
     return ret.ApplicationId;
@@ -583,7 +611,7 @@ QString ApplicationManager1Service::Identify(const QDBusUnixFileDescriptor &pidf
 void ApplicationManager1Service::updateApplication(const QSharedPointer<ApplicationService> &destApp,
                                                    DesktopFile desktopFile) noexcept
 {
-    if (auto app = m_applicationList.find(destApp->applicationPath()); app == m_applicationList.cend()) {
+    if (!m_applicationList.contains(destApp->id())) {
         return;
     }
 
@@ -605,29 +633,29 @@ void ApplicationManager1Service::updateApplication(const QSharedPointer<Applicat
     }
 
     if (destApp->m_desktopSource != desktopFile and destApp->isAutoStart()) {
-        destApp->setAutoStart(false);
         destApp->m_desktopSource = std::move(desktopFile);
-        destApp->setAutoStart(true);
     }
 }
 
 void ApplicationManager1Service::ReloadApplications()
 {
-    if (m_reloadTimer.isActive()) {
-        qInfo() << "reloadTimer is running, ignore...";
+    if (calledFromDBus() && !m_reloadTimer.isActive()) {
+        doReloadApplications();
         return;
     }
-
     m_reloadTimer.start();
+}
+
+void ApplicationManager1Service::doReloadApplications()
+{
+    qInfo() << "reload applications.";
 
     auto desktopFileDirs = getDesktopFileDirs();
-    desktopFileDirs.append(getAutoStartDirs());  // detect autostart apps add/remove/update
-
-    auto apps = m_applicationList.keys();
+    auto appIds = m_applicationList.keys();
 
     applyIteratively(
         QList<QDir>(desktopFileDirs.cbegin(), desktopFileDirs.cend()),
-        [this, &apps](const QFileInfo &info) -> bool {
+        [this, &appIds](const QFileInfo &info) -> bool {
             ParserError err{ParserError::NoError};
             auto ret = DesktopFile::searchDesktopFileByPath(info.absoluteFilePath(), err);
             if (!ret.has_value()) {
@@ -635,20 +663,15 @@ void ApplicationManager1Service::ReloadApplications()
             }
 
             auto file = std::move(ret).value();
-
-            auto destApp =
-                std::find_if(m_applicationList.cbegin(),
-                             m_applicationList.cend(),
-                             [&file](const QSharedPointer<ApplicationService> &app) { return file.desktopId() == app->id(); });
-
-            if (err != ParserError::NoError) {
-                qWarning() << "error occurred:" << err << " skip this application.";
-                return false;
-            }
-
-            if (destApp != m_applicationList.cend() and apps.contains(destApp.key())) {
-                apps.removeOne(destApp.key());
-                updateApplication(destApp.value(), std::move(file));
+            auto app = m_applicationList.value(file.desktopId());
+            if (app && appIds.contains(app->id())) {
+                // Can emit correct remove signal when uninstalling applications
+                if (ApplicationFilter::tryExecCheck(*(app.data()->m_entry))) {
+                    qDebug() << info.absolutePath() << "Checked TryExec failed and will be removed";
+                    return false;
+                }
+                appIds.removeOne(app->id());
+                updateApplication(app, std::move(file));
                 return false;
             }
 
@@ -659,13 +682,12 @@ void ApplicationManager1Service::ReloadApplications()
         {"*.desktop"},
         QDir::Name | QDir::DirsLast);
 
-    for (const auto &key : apps) {
-        removeOneApplication(key);
+    for (const auto &appId : appIds) {
+        removeOneApplication(appId);
     }
 
     m_mimeManager->reset();
     scanMimeInfos();
-
     scanAutoStart();
 }
 
@@ -674,14 +696,13 @@ ObjectMap ApplicationManager1Service::GetManagedObjects() const
     return dumpDBusObject(m_applicationList);
 }
 
-QMap<QDBusObjectPath, QSharedPointer<ApplicationService>>
+QHash<QDBusObjectPath, QSharedPointer<ApplicationService>>
 ApplicationManager1Service::findApplicationsByIds(const QStringList &appIds) const noexcept
 {
-    QMap<QDBusObjectPath, QSharedPointer<ApplicationService>> ret;
-    for (auto it = m_applicationList.constKeyValueBegin(); it != m_applicationList.constKeyValueEnd(); ++it) {
-        const auto &ptr = it->second;
-        if (appIds.contains(ptr->id())) {
-            ret.insert(it->first, it->second);
+    QHash<QDBusObjectPath, QSharedPointer<ApplicationService>> ret;
+    for (const auto &appId : appIds) {
+        if (auto app = m_applicationList.value(appId); app) {
+            ret.insert(QDBusObjectPath{getObjectPathFromAppId(appId)}, app);
         }
     }
 
@@ -691,43 +712,63 @@ ApplicationManager1Service::findApplicationsByIds(const QStringList &appIds) con
 QString ApplicationManager1Service::addUserApplication(const QVariantMap &desktop_file, const QString &name) noexcept
 {
     if (name.isEmpty()) {
-        sendErrorReply(QDBusError::Failed, "file name is empty.");
+        safe_sendErrorReply(QDBusError::Failed, "file name is empty.");
+        return {};
+    }
+
+    // 传入的desktop文件内容为空的时候直接返回。
+    QString errMsg;
+    auto fileContent = DesktopFileGenerator::generate(desktop_file, errMsg);
+    if (fileContent.isEmpty() or !errMsg.isEmpty()) {
+        safe_sendErrorReply(QDBusError::Failed, errMsg);
         return {};
     }
 
     QDir xdgDataHome;
     QString dir{getXDGDataHome() + "/applications"};
     if (!xdgDataHome.mkpath(dir)) {
-        sendErrorReply(QDBusError::Failed, "couldn't create directory of user applications.");
+        safe_sendErrorReply(QDBusError::Failed, "couldn't create directory of user applications.");
         return {};
     }
 
+    // 判断当前是否已经存在了desktop文件
     xdgDataHome.setPath(dir);
     const auto &filePath = xdgDataHome.filePath(name);
-
-    if (QFileInfo info{filePath}; info.exists() and info.isFile()) {
-        sendErrorReply(QDBusError::Failed, QString{"file already exists:%1"}.arg(info.absoluteFilePath()));
-        return {};
-    }
-
     QFile file{filePath};
-    if (!file.open(QFile::NewOnly | QFile::WriteOnly | QFile::Text)) {
-        sendErrorReply(QDBusError::Failed, file.errorString());
-        return {};
+    const auto fileExists = file.exists();
+    bool shouldRewrite = false;
+
+    if (fileExists) {
+        ParserError parseErr{ParserError::NoError};
+        // 判断desktop文件内容是否合法
+        auto existingDesktopFile = DesktopFile::searchDesktopFileByPath(filePath, parseErr);
+        if (existingDesktopFile) {
+            // 解析当前已经存在的desktop文件内容
+            DesktopEntry entry;
+            ParserError entryErr = entry.parse(existingDesktopFile.value());
+            if (entryErr != ParserError::NoError) {
+                qWarning() << "parse existing desktop file has error:" << entryErr;
+                shouldRewrite = true;
+            }
+        }
+    }
+    // 根据情况选择打开模式
+    QIODevice::OpenMode openMode = QFile::WriteOnly | QFile::Text;
+    if (fileExists && shouldRewrite) {
+        openMode |= QFile::Truncate;  // 清空现有内容
+    } else if (!fileExists) {
+        openMode |= QFile::NewOnly;  // 只创建新文件
     }
 
-    QString errMsg;
-    auto fileContent = DesktopFileGenerator::generate(desktop_file, errMsg);
-    if (fileContent.isEmpty() or !errMsg.isEmpty()) {
-        file.remove();
-        sendErrorReply(QDBusError::Failed, errMsg);
+    if (!file.open(openMode)) {
+        safe_sendErrorReply(QDBusError::Failed, file.errorString());
         return {};
     }
 
     auto writeContent = fileContent.toLocal8Bit();
     if (file.write(writeContent) != writeContent.size()) {
         file.remove();
-        sendErrorReply(QDBusError::Failed, "incomplete file content.this file will be removed.");
+        safe_sendErrorReply(QDBusError::Failed, "incomplete file content.this file will be removed.");
         return {};
     }
 
@@ -738,13 +779,13 @@ QString ApplicationManager1Service::addUserApplication(const QVariantMap &deskto
     if (err != ParserError::NoError) {
         file.remove();
         qDebug() << "add user's application failed:" << err;
-        sendErrorReply(QDBusError::Failed, "search failed.");
+        safe_sendErrorReply(QDBusError::Failed, "search failed.");
         return {};
     }
 
     if (!ret) {
         file.remove();
-        sendErrorReply(QDBusError::InternalError);
+        safe_sendErrorReply(QDBusError::InternalError);
         return {};
     }
 
@@ -752,9 +793,50 @@ QString ApplicationManager1Service::addUserApplication(const QVariantMap &deskto
     auto appId = desktopSource.desktopId();
     if (!addApplication(std::move(desktopSource))) {
         file.remove();
-        sendErrorReply(QDBusError::Failed, "add application to ApplicationManager failed.");
+        safe_sendErrorReply(QDBusError::Failed, "add application to ApplicationManager failed.");
         return {};
     }
 
+    m_mimeManager->updateMimeCache(dir);
     return appId;
 }
+
+void ApplicationManager1Service::deleteUserApplication(const QString &app_id) noexcept
+{
+    if (app_id.isEmpty()) {
+        safe_sendErrorReply(QDBusError::Failed, "app id name is empty.");
+        return;
+    }
+
+    QDir xdgDataHome;
+    QString dir{getXDGDataHome() + "/applications"};
+    if (!xdgDataHome.mkpath(dir)) {
+        safe_sendErrorReply(QDBusError::Failed, "couldn't create directory of user applications.");
+        return;
+    }
+
+    xdgDataHome.setPath(dir);
+    const auto &filePath = xdgDataHome.filePath(app_id + ".desktop");
+
+    if (QFileInfo info{filePath}; !info.exists() || !info.isFile()) {
+        safe_sendErrorReply(QDBusError::Failed, QString{"file not exists:%1"}.arg(info.absoluteFilePath()));
+        return;
+    }
+
+    /*
+        auto apps = findApplicationsByIds({app_id});
+        if (apps.isEmpty()) {
+            qWarning() << "we can't find corresponding application in ApplicationManagerService.";
+        }
+
+        for (const auto &app : apps) {
+            app->setMimeTypes({});
+        }
+    */
+    if (!QFile::remove(filePath)) {
+        safe_sendErrorReply(QDBusError::Failed, "remove file failed.");
+        return;
+    }
+
+    m_mimeManager->updateMimeCache(dir);
+}
